﻿using System;
using System.ComponentModel;
using System.Linq;
using System.Windows.Forms;
using C1.Win.C1FlexGrid;

namespace FlexGridSearch
{
    public partial class C1FlexGridSearchPanel : UserControl
    {
        [DefaultValue(SearchHighlightMode.OnlyFirst)]
        public SearchHighlightMode HighlightSearchResultsMode { get; set; } = SearchHighlightMode.OnlyFirst;

        [DefaultValue(true)]
        public bool HighlightSearchResults { get; set; } = true;

        [DefaultValue(true)]
        public bool FilterSearchResults { get; set; } = true;

        [DefaultValue(true)]
        public bool SearchThroughVisibleCols { get; set; } = true;

        [DefaultValue(true)]
        public bool SearchThroughVisibleRows { get; set; } = true;

        [DefaultValue(1000)]
        public int SearchDelay { get { return _timer.Interval; } set { _timer.Interval = value; } }

        private SearchMode _searchMode = SearchMode.SearchClick;

        [DefaultValue(SearchMode.SearchClick)]
        public SearchMode SearchMode
        {
            get { return _searchMode; }
            set
            {
                _searchMode = value;
                btnSearch.Visible = _searchMode == SearchMode.SearchClick;
            }
        }

        private bool IsBound { get { return _grid != null; } }

        private C1FlexGrid _grid;
        private readonly string _searchDefinition = "<SearchDefinition />";
        private readonly Timer _timer = new Timer() { Interval = 1000 };


        public C1FlexGridSearchPanel()
        {
            InitializeComponent();

            btnSearch.Image = Properties.Resources.Search;
            btnCancel.Image = Properties.Resources.Cancel;

            _timer.Tick += _timer_Tick;
            comboBox1.TextChanged += ComboBox1_TextChanged;
        }

        public void SetC1FlexGridSearchPanel(C1FlexGrid c1flexGrid)
        {
            _grid = c1flexGrid;
        }

        private void _timer_Tick(object sender, EventArgs e)
        {
            if (string.IsNullOrWhiteSpace(comboBox1.Text))
            {
                ClearSearch();
                return;
            }
            Searchgrid();
            _timer.Stop();
        }
        private void ComboBox1_TextChanged(object sender, EventArgs e)
        {
            if (IsBound && SearchMode == SearchMode.Always)
            {
                _timer.Start();
            }
        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            if (IsBound) Searchgrid();
        }
        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (IsBound) ClearSearch();
        }

        private void Searchgrid()
        {
            if (_grid != null && !string.IsNullOrEmpty(comboBox1.Text) && !string.IsNullOrWhiteSpace(comboBox1.Text))
            {
                //include search items in "" internally
                var wordsSpecific = comboBox1.Text.Split(' ').Select(x => string.Format("\"{0}\"", x)).ToList();
                
                _grid.ApplySearch(string.Join(", ", wordsSpecific), HighlightSearchResultsMode, FilterSearchResults, SearchThroughVisibleCols, SearchThroughVisibleRows);
                
                if (!comboBox1.Items.Contains(comboBox1.Text))
                {
                    comboBox1.Items.Add(comboBox1.Text);
                }
            }
            if (string.IsNullOrWhiteSpace(comboBox1.Text))
            {
                ClearSearch();
            }
        }
        private void ClearSearch()
        {
            if (_grid != null && _grid.SearchDefinition != _searchDefinition)
            {
                _grid.SearchDefinition = _searchDefinition;
                comboBox1.Text = string.Empty;
            }
        }
    }

    public enum SearchMode
    {
        Always,
        SearchClick
    }
}
